<?php

/**
 * Class SupsysticSlider_Ui_Module
 * User Interface Module
 *
 * @package SupsysticSlider\Ui
 * @author Artur Kovalevsky
 */
class SupsysticSlider_Ui_Module extends Rsc_Mvc_Module
{
    /**
     * @var array
     */
    protected $javascripts;

    /**
     * @var array
     */
    protected $stylesheets;

    /**
     * @var SupsysticSlider_Ui_AssetsCollection
     */
    protected $assets;

    /**
     * {@inheritdoc}
     */
    public function onInit()
    {
        parent::onInit();

        $this->assets = new SupsysticSlider_Ui_AssetsCollection();

        $this->preload();

        $config = $this->getEnvironment()->getConfig();

        add_action(
            $config->get('hooks_prefix') . 'after_modules_loaded',
            array($this->assets, 'load')
        );

        add_action('admin_enqueue_scripts', array($this, 'menuAntiDuplicate'));
        add_action('admin_enqueue_scripts', array($this, 'colorpicker'));

        // Allows to sort menu items.
        $dispatcher = $this->getEnvironment()->getDispatcher();

        $dispatcher->dispatch('ui_menu_items');
        $this->getEnvironment()->getMenu()->register();
    }

    public function menuAntiDuplicate()
    {
        $url = $this->getEnvironment()->getConfig()->get('plugin_url');
        wp_enqueue_style('rs-menu-anti-duplicate', $url . '/app/assets/css/supsystic-for-all-admin.css');
    }

    public function colorpicker()
    {
        if ($this->isPluginPage()) {
            wp_enqueue_style('wp-color-picker');
            wp_enqueue_script('rs-color-picker', $this->getLocationUrl() . '/js/colorpicker.js', array('wp-color-picker'));
        }
    }

    /**
     * Adds the asset
     * @param SupsysticSlider_Ui_Asset $asset
     */
    public function add(SupsysticSlider_Ui_Asset $asset)
    {
        $this->assets->add($asset);
    }

    /**
     * Returns the asset if it exists.
     * @param string $handle
     * @param mixed $default
     * @return SupsysticSlider_Ui_Asset
     */
    public function get($handle, $default = null)
    {
        return $this->assets->get($handle, $default);
    }

    /**
     * Deletes the asset.
     * @param string $handle
     * @return bool
     */
    public function delete($handle)
    {
        return $this->assets->delete($handle);
    }

    /**
     * Preloads the assets
     */
    protected function preload()
    {
        /* URL to the plugin folder */
        $url = $this->getEnvironment()->getConfig()->get('plugin_url');

        /* CSS */
        $this->add(new SupsysticSlider_Ui_BackendStylesheet('rs-ui', $url . '/app/assets/css/supsystic-ui.css'));
        $this->add(new SupsysticSlider_Ui_BackendStylesheet('jqgrid-ui', $url . '/app/assets/css/ui.jqgrid.css'));
        $this->add(new SupsysticSlider_Ui_BackendStylesheet('rs-jquery-min-ui', $url.'/app/assets/css/jquery-ui.min.css'));
        $this->add(new SupsysticSlider_Ui_BackendStylesheet('rs-jquery-structure-min-ui', $url.'/app/assets/css/jquery-ui.structure.min.css'));
        $this->add(new SupsysticSlider_Ui_BackendStylesheet('rs-jquery-theme-min-ui', $url.'/app/assets/css/jquery-ui.theme.min.css'));
		$this->add(new SupsysticSlider_Ui_BackendStylesheet('rs-font-awesome', $url.'/app/assets/css/font-awesome-4-7-0.css'));
		$this->add(new SupsysticSlider_Ui_BackendStylesheet('rs-jgrowl', $url . '/app/assets/css/supsystic-jgrowl.css'));
		$this->add(new SupsysticSlider_Ui_BackendStylesheet('rs-tooltipster', $url . '/app/assets/css/tooltipster3-3-0.css'));
		$this->add(new SupsysticSlider_Ui_BackendStylesheet('rs-tooltipster-theme-shadow', $url . '/app/assets/css/tooltipster-shadow3-3-0.css'));
        $this->add(new SupsysticSlider_Ui_BackendStylesheet('rs-shadows-backend-css', $url . '/app/assets/css/shadows.css'));
		$this->add(new SupsysticSlider_Ui_BackendStylesheet('rs-minimal', $url . '/app/assets/css/minimal/minimal.css'));

        /* Javascript */
        $this->add(new SupsysticSlider_Ui_BackendJavascript('jquery'));
        $this->add(new SupsysticSlider_Ui_BackendJavascript('jquery-ui-dialog'));
		$this->add(new SupsysticSlider_Ui_BackendJavascript('rs-icheck', $url . '/app/assets/js/icheck.min.js'));
		$this->add(new SupsysticSlider_Ui_BackendJavascript('rs-types', $this->getLocationUrl() . '/js/types.js'));
        $this->add(new SupsysticSlider_Ui_BackendJavascript('rs-ui-js', $url . '/app/assets/js/gird-gallery.js'));
        $this->add(new SupsysticSlider_Ui_BackendJavascript('rs-lazy-load-ks', $url . '/app/assets/js/jquery.lazyload.min.js'));
        $this->add(new SupsysticSlider_Ui_BackendJavascript('rs-form-serializer-js', $this->getLocationUrl() . '/plugins/gird-gallery.ui.formSerialize.js'));
		$this->add(new SupsysticSlider_Ui_BackendJavascript('rs-jgrowl-js', $this->getLocationUrl() . '/js/jquery.jgrowl-1-2-12.min.js'));
		$this->add(new SupsysticSlider_Ui_BackendJavascript('rs-tooltipster-js', $this->getLocationUrl() . '/js/jquery.tooltipster3-3-0.min.js'));
		$this->add(new SupsysticSlider_Ui_BackendJavascript('gg-ddslick-js', $this->getLocationUrl() . '/js/jquery.ddslick2-0.min.js'));
        $this->add(new SupsysticSlider_Ui_BackendJavascript('rs-slimscroll-js', $this->getLocationUrl() . '/js/jquery.slimscroll.min.js'));
        $this->add(new SupsysticSlider_Ui_BackendJavascript('rs-jqgrid-min-js', $this->getLocationUrl() . '/js/jquery.jqGrid4-6-0.min.js'));
        $this->add(new SupsysticSlider_Ui_BackendJavascript('rs-jqgrid-locale-en-js', $this->getLocationUrl() . '/js/jqgrid-4.6.0-i18n-grid.locale-en.js'));
        $this->add(new SupsysticSlider_Ui_BackendJavascript('rs-toolbar-js', $this->getLocationUrl() . '/plugins/gird-gallery.ui.toolbar.js'));
        $this->add(new SupsysticSlider_Ui_BackendJavascript('rs-cb-observ', $this->getLocationUrl() . '/js/checkbox-observer.js'));
        $this->add(new SupsysticSlider_Ui_BackendJavascript('rs-ui-toolbar', $this->getLocationUrl() . '/js/toolbar.js'));
        $this->add(new SupsysticSlider_Ui_BackendJavascript('rs-common', $this->getLocationUrl() . '/js/common.js'));
        $this->add(new SupsysticSlider_Ui_BackendJavascript('rs-ajax', $this->getLocationUrl() . '/js/ajax.js'));
        $this->add(new SupsysticSlider_Ui_BackendJavascript('rs-ajax-queue', $this->getLocationUrl() . '/js/ajaxQueue.js'));
		$this->add(new SupsysticSlider_Ui_BackendJavascript('rs-holder', $url . '/app/assets/js/holder.js'));
    }

}
